//////////////////////////////////////////////////////////////////////////////
//
// "Embedded C Programming: Techniques and Applications of C and PIC MCUs"
// by Mark Siegusmund
//
// Exercise 19-1 #2
//
// Displays the frequency of an incoming signal on Pin C5.
//
//////////////////////////////////////////////////////////////////////////////

#include <e3.h>
#include <ios.h>

#include <tick.c>

enum states {WAIT_FOR_L2H, WAIT_FOR_H2L};
states state = WAIT_FOR_L2H;
int16 time[10];
int count=0;

#int_ccp1
void capture_isr(void) {
   static int16 rise;
   
   switch(state) {
      case WAIT_FOR_L2H :
         rise=CCP_1;
         setup_ccp1(CCP_CAPTURE_FE);
         state=WAIT_FOR_H2L;
         break;
      case WAIT_FOR_H2L :
         time[count++]=CCP_1-rise;
         setup_ccp1(CCP_CAPTURE_RE);
         state=WAIT_FOR_L2H;
         if(count==10)
            count=0;
         break;
   }
}

float calcFreq(void) {
   int32 sum;
   
   for(int i = 0; i < 10; i++)
   {
     sum+=time[i]; 
   }
   
   // sum increments at 12Mhz rate (see Timer 1), but is only
   // the measurement of half of a square wave period.
   
   return (float)60000000 / sum;
}

void main(void) {
   float freq;

   memset(time, 0, sizeof(time));
   
   setup_timer_1(T1_INTERNAL | T1_DIV_BY_1);   //ticks at 12mhz rate
   setup_ccp1(CCP_CAPTURE_RE);
   state=WAIT_FOR_L2H;
   
   clear_interrupt(INT_CCP1);
   enable_interrupts(INT_CCP1);
   enable_interrupts(GLOBAL);
   
   for(;;) {
      TickDelay(TICKS_PER_SECOND);
      
      disable_interrupts(INT_CCP1);
      
      freq = calcFreq();
      
      cout << "Frequency is " << setprecision(1) << freq << "hz" << endl;
      
      enable_interrupts(INT_CCP1);
   }
   
   for(;;);
}
